void W5100_Hardware_Init()
{
  W5100_SCS=1;
  W5100_RESET=1;
  W5100_SCS_DIRECTION=0;
  W5100_RESET_DIRECTION=0;
  SPI1_Init_Advanced(_SPI_MASTER_OSC_DIV4,_SPI_DATA_SAMPLE_MIDDLE,_SPI_CLK_IDLE_LOW,_SPI_LOW_2_HIGH);
  W5100_SCS=1;
  W5100_RESET=1;
  delay_ms(10);//Wait For Spi To Init Some Ms's

  W5100_RESET=0;
  delay_us(2);//2 us for reset
  W5100_RESET=1;
  delay_ms(10);//10 ms until reset occurs
}


void W5100_Write_Byte(unsigned int Address,unsigned short DATA)
{
  unsigned char HighAddress;
  unsigned char LowAddress;
  HighAddress=(Address&0xff00)>>8;
  LowAddress=Address&0x00ff;
  W5100_SCS=0;
  SPI1_Write(WIZNET_WRITE_OPCODE);
  SPI1_Write(HighAddress);
  SPI1_Write(LowAddress);
  SPI1_Write(DATA);
  W5100_SCS=1;
}

unsigned char W5100_Read_Byte(unsigned int Address)
{
  unsigned char HighAddress;
  unsigned char LowAddress;
  char Val;
  HighAddress=(Address&0xff00)>>8;
  LowAddress=Address&0x00ff;

  W5100_SCS=0;
  SPI1_Write(WIZNET_READ_OPCODE);
  SPI1_Write(HighAddress);
  SPI1_Write(LowAddress);
  Val=SPI1_Read(0);
  W5100_SCS=1;
  return val;
}

void W5100_Init(char* IpAddress, char* Subnet, char* Gatway, char* MacAddress)
{
  W5100_Write_Byte(MODE_REGISTER,0x80);//SOFTWARE RESET

  W5100_Write_Byte(GATE_WAY_ADDRESS,Gatway[0]);
  W5100_Write_Byte(GATE_WAY_ADDRESS+1,Gatway[1]);
  W5100_Write_Byte(GATE_WAY_ADDRESS+2,Gatway[2]);
  W5100_Write_Byte(GATE_WAY_ADDRESS+3,Gatway[3]);

  W5100_Write_Byte(MAC,MacAddress[0]);
  W5100_Write_Byte(MAC+1,MacAddress[1]);
  W5100_Write_Byte(MAC+2,MacAddress[2]);
  W5100_Write_Byte(MAC+3,MacAddress[3]);
  W5100_Write_Byte(MAC+4,MacAddress[4]);
  W5100_Write_Byte(MAC+5,MacAddress[5]);

  W5100_Write_Byte(SUBNET_MASK,Subnet[0]);
  W5100_Write_Byte(SUBNET_MASK+1,Subnet[1]);
  W5100_Write_Byte(SUBNET_MASK+2,Subnet[2]);
  W5100_Write_Byte(SUBNET_MASK+3,Subnet[3]);

  W5100_Write_Byte(DEVICE_IP,IpAddress[0]);
  W5100_Write_Byte(DEVICE_IP+1,IpAddress[1]);
  W5100_Write_Byte(DEVICE_IP+2,IpAddress[2]);
  W5100_Write_Byte(DEVICE_IP+3,IpAddress[3]);

  W5100_Write_Byte(RX_MEMORY_SIZE_REGISTER,RX_TX_MALLOC);
  W5100_Write_Byte(TX_MEMORY_SIZE_REGISTER,RX_TX_MALLOC);
}

void CloseSocket()
{
  //send command close to control register
  W5100_Write_Byte(SOCKET0_CONTROL_REGISTER,SOCKET_CONTROL_REGISTER_CLOSE);
  //wait untill control register is zero
  while(W5100_Read_Byte(SOCKET0_CONTROL_REGISTER));
}

void DisconnectSocket()
{
   //send command Disconnect to control register
   W5100_Write_Byte(SOCKET0_CONTROL_REGISTER,SOCKET_CONTROL_REGISTER_DISCONNECT);
   // wait untill control register is zero
   while(W5100_Read_Byte(SOCKET0_CONTROL_REGISTER));
}

unsigned char ReadStatusRegister()
{
  return W5100_Read_Byte(SOCKET0_STATUS_REGISTER);
}

unsigned char IsSocketClosed()
{
  return (W5100_Read_Byte(SOCKET0_STATUS_REGISTER) == SOCKET_CLOSED_STATUS);
}

unsigned char IsSocketInitialized()
{
  return (W5100_Read_Byte(SOCKET0_STATUS_REGISTER) == SOCKET_INIT_STATUS);
}

unsigned char IsSocketListening()
{
  return (W5100_Read_Byte(SOCKET0_STATUS_REGISTER) == SOCKET_LISTEN_STATUS);
}




unsigned char InitSocket(unsigned int PORT)
{
    unsigned char Error;
    unsigned char HighByte;
    unsigned char LowByte;

    Error=1;
    //force close
    if (IsSocketClosed())CloseSocket();

    // assign socket0 with protocol
    W5100_Write_Byte(SOCKET0_MODE_REGISTER,SOCKET0_MODE_REGISTER_TCP);

    //give port to socket
    HighByte=(PORT&0xff00)>>8;
    LowByte=(PORT&0x00ff);
    W5100_Write_Byte(SOCKET0_LOCAL_PORT,HighByte);
    W5100_Write_Byte(SOCKET0_LOCAL_PORT+1,LowByte);
    //open the socket
    W5100_Write_Byte(SOCKET0_CONTROL_REGISTER,SOCKET_CONTROL_REGISTER_OPEN);

    // wait untill reg is zero
    while(W5100_Read_Byte(SOCKET0_CONTROL_REGISTER));

    //check if socket is initialized well
    if (IsSocketInitialized())
    Error=0;
    else
    CloseSocket();

    return Error;
}


unsigned char StartListening()
{
   unsigned char Error;
   Error=1;

   if (IsSocketInitialized)
   {
   // Send the LISTEN Command
   W5100_Write_Byte(SOCKET0_CONTROL_REGISTER,SOCKET_CONTROL_REGISTER_LISTEN);
   // wait untill reg is zero
   while(W5100_Read_Byte(SOCKET0_CONTROL_REGISTER));
   // Check for Listen Status
   if (IsSocketListening())
   Error=0;
   else
   CloseSocket();
   }
   return Error;
}

void W5100_WriteTxBuffer(unsigned int Address, char* Data, unsigned int Size)
{
  unsigned int i;
  for(i=0;i<Size;i++,Address++)
  {
    W5100_Write_Byte(Address,Data[i]);
  }
}

void W5100_ConstWriteTxBuffer(unsigned int Address, const char* Data, unsigned int Size)
{
  unsigned int i;
  for(i=0;i<Size;i++,Address++)
  {
    W5100_Write_Byte(Address,Data[i]);
  }
}

void W5100_ReadRxBuffer(unsigned int Address, char* Destination, unsigned int Size)
{
 unsigned int i;
 for(i=0;i<Size;i++,Address++)
 {
   Destination[i]=W5100_Read_Byte(Address);
 }
}

unsigned char Write_Page(char* TxBuffer, unsigned int SendSize)
{

unsigned int FreeSize;

//unsigned int TimeOut;
unsigned int MemoryPointer;
unsigned int PhisycalAddress;
unsigned char HighByte;
unsigned char LowByte;
unsigned int GetOffset;//

// Read the Tx Write Pointer
MemoryPointer =(unsigned int)W5100_Read_Byte(SOCKET0_TX_WRITE_PTR);
MemoryPointer=MemoryPointer<<8;
MemoryPointer=MemoryPointer|(unsigned int)W5100_Read_Byte(SOCKET0_TX_WRITE_PTR+1);

GetOffset=MemoryPointer&TX_BUF_MASK;
PhisycalAddress=BASE_TX_BUFFER+GetOffset;

if(GetOffset+SendSize>TX_BUF_MASK+1)
{
  FreeSize=(TX_BUF_MASK+1)-GetOffset;
  W5100_WriteTxBuffer(PhisycalAddress,TxBuffer,FreeSize);
  W5100_WriteTxBuffer(BASE_TX_BUFFER,TxBuffer+FreeSize,SendSize-FreeSize);
}
else
{
  W5100_WriteTxBuffer(PhisycalAddress,TxBuffer,SendSize);
}

MemoryPointer+=SendSize;


// assign the tx write pointer , so it point to the next transmit
HighByte=(MemoryPointer&0xff00)>>8;
LowByte=MemoryPointer&0x00ff;
W5100_Write_Byte(SOCKET0_TX_WRITE_PTR,HighByte);
W5100_Write_Byte(SOCKET0_TX_WRITE_PTR+1,LowByte);

return 0;
}

unsigned char Write_ConstPage(const char* TxBuffer, unsigned int SendSize)
{

unsigned int FreeSize;

//unsigned int TimeOut;
unsigned int MemoryPointer;
unsigned int PhisycalAddress;
unsigned char HighByte;
unsigned char LowByte;
unsigned int GetOffset;//

// Read the Tx Write Pointer
MemoryPointer =(unsigned int)W5100_Read_Byte(SOCKET0_TX_WRITE_PTR);
MemoryPointer=MemoryPointer<<8;
MemoryPointer=MemoryPointer|(unsigned int)W5100_Read_Byte(SOCKET0_TX_WRITE_PTR+1);

GetOffset=MemoryPointer&TX_BUF_MASK;
PhisycalAddress=BASE_TX_BUFFER+GetOffset;

if(GetOffset+SendSize>TX_BUF_MASK+1)
{
  FreeSize=(TX_BUF_MASK+1)-GetOffset;
  W5100_ConstWriteTxBuffer(PhisycalAddress,TxBuffer,FreeSize);
  W5100_ConstWriteTxBuffer(BASE_TX_BUFFER,TxBuffer+FreeSize,SendSize-FreeSize);
}
else
{
  W5100_ConstWriteTxBuffer(PhisycalAddress,TxBuffer,SendSize);
}

MemoryPointer+=SendSize;


// assign the tx write pointer , so it point to the next transmit
HighByte=(MemoryPointer&0xff00)>>8;
LowByte=MemoryPointer&0x00ff;
W5100_Write_Byte(SOCKET0_TX_WRITE_PTR,HighByte);
W5100_Write_Byte(SOCKET0_TX_WRITE_PTR+1,LowByte);

return 0;
}



unsigned char Get_Request(char* ReceiveBuffer, unsigned int ReceiveSize)
{
unsigned int GetOffset;
unsigned int MemoryPointer;
unsigned int PhisycalAddress;
unsigned char HighByte;
unsigned char LowByte;
unsigned int FreeSize;



// Read the Rx Write Pointer
MemoryPointer =(unsigned int)W5100_Read_Byte(SOCKET0_RX_READ_PTR);
MemoryPointer=MemoryPointer<<8;
MemoryPointer=MemoryPointer|(unsigned int)W5100_Read_Byte(SOCKET0_RX_READ_PTR+1);

GetOffset=MemoryPointer&RX_BUF_MASK;
PhisycalAddress=BASE_RX_BUFFER+GetOffset;

if(GetOffset+ReceiveSize>RX_BUF_MASK+1)
{
  FreeSize=(RX_BUF_MASK+1)-GetOffset;
  W5100_ReadRxBuffer(PhisycalAddress,ReceiveBuffer,FreeSize);
  W5100_ReadRxBuffer(BASE_RX_BUFFER,ReceiveBuffer+FreeSize,ReceiveSize-FreeSize);
}
else
{
  W5100_ReadRxBuffer(PhisycalAddress,ReceiveBuffer,ReceiveSize);
}

MemoryPointer+=ReceiveSize;

// assign the rx  pointer , so it point to the next transmit
HighByte=(MemoryPointer&0xff00)>>8;
LowByte=MemoryPointer&0x00ff;
W5100_Write_Byte(SOCKET0_RX_READ_PTR,HighByte);
W5100_Write_Byte(SOCKET0_RX_READ_PTR+1,LowByte);

//command recive to control register
W5100_Write_Byte(SOCKET0_CONTROL_REGISTER,SOCKET_CONTROL_REGISTER_RECEIVE);
delay_us(10);

ReceiveBuffer[ReceiveSize-1]='\0';

return 0;
}

unsigned int GetRequestSize()
{
  unsigned int Size;
  Size=0;
  Size=(unsigned int)W5100_Read_Byte(SOCKET0_RECEIVESIZE_REGISTER);
  Size=Size<<8;
  Size=Size|(unsigned int)W5100_Read_Byte(SOCKET0_RECEIVESIZE_REGISTER+1);
  return Size;
}


unsigned int GetLocalPort()
{
  unsigned int Val;
  Val=(unsigned int)W5100_Read_Byte(SOCKET0_LOCAL_PORT);
  Val=Val<<8;
  Val=Val|(unsigned int)W5100_Read_Byte(SOCKET0_LOCAL_PORT+1);
  return Val;
}

unsigned int GetRemotePort()
{
  unsigned int Val;
  Val=(unsigned int)W5100_Read_Byte(SOCKET0_DESTINATION_PORT);
  Val=Val<<8;
  Val=Val|(unsigned int)W5100_Read_Byte(SOCKET0_DESTINATION_PORT+1);
  return Val;
}

char* GetRemoteIp()
{
_Remote_Ip_Address_[0]=W5100_Read_Byte(SOCKET0_DESTINATION_IP);
_Remote_Ip_Address_[1]=W5100_Read_Byte(SOCKET0_DESTINATION_IP+1);
_Remote_Ip_Address_[2]=W5100_Read_Byte(SOCKET0_DESTINATION_IP+2);
_Remote_Ip_Address_[3]=W5100_Read_Byte(SOCKET0_DESTINATION_IP+3);
return _Remote_Ip_Address_;
}

char* GetRemoteMac()
{
_Remote_Mac_Address_[0]=W5100_Read_Byte(SOCKET0_DESTINATION_MAC);
_Remote_Mac_Address_[1]=W5100_Read_Byte(SOCKET0_DESTINATION_MAC+1);
_Remote_Mac_Address_[2]=W5100_Read_Byte(SOCKET0_DESTINATION_MAC+2);
_Remote_Mac_Address_[3]=W5100_Read_Byte(SOCKET0_DESTINATION_MAC+3);
_Remote_Mac_Address_[4]=W5100_Read_Byte(SOCKET0_DESTINATION_MAC+4);
_Remote_Mac_Address_[5]=W5100_Read_Byte(SOCKET0_DESTINATION_MAC+5);
return _Remote_Mac_Address_;
}

char* GetLocal_IpAddress()
{
  _Remote_Ip_Address_[0]=W5100_Read_Byte(DEVICE_IP);
  _Remote_Ip_Address_[1]=W5100_Read_Byte(DEVICE_IP+1);
  _Remote_Ip_Address_[2]=W5100_Read_Byte(DEVICE_IP+2);
  _Remote_Ip_Address_[3]=W5100_Read_Byte(DEVICE_IP+3);
  return _Remote_Ip_Address_;
}

char* GetLocal_MacAddress()
{
  _Remote_Mac_Address_[0]=W5100_Read_Byte(MAC);
  _Remote_Mac_Address_[1]=W5100_Read_Byte(MAC+1);
  _Remote_Mac_Address_[2]=W5100_Read_Byte(MAC+2);
  _Remote_Mac_Address_[3]=W5100_Read_Byte(MAC+3);
  _Remote_Mac_Address_[4]=W5100_Read_Byte(MAC+4);
  _Remote_Mac_Address_[5]=W5100_Read_Byte(MAC+5);
  return _Remote_Mac_Address_;
}

char* GetLocal_SubnetMask()
{
  _Remote_Ip_Address_[0]=W5100_Read_Byte(SUBNET_MASK);
  _Remote_Ip_Address_[1]=W5100_Read_Byte(SUBNET_MASK+1);
  _Remote_Ip_Address_[2]=W5100_Read_Byte(SUBNET_MASK+2);
  _Remote_Ip_Address_[3]=W5100_Read_Byte(SUBNET_MASK+3);
  return _Remote_Ip_Address_;
}

char* GetLocal_DefaultGateway()
{
  _Remote_Ip_Address_[0]=W5100_Read_Byte(GATE_WAY_ADDRESS);
  _Remote_Ip_Address_[1]=W5100_Read_Byte(GATE_WAY_ADDRESS+1);
  _Remote_Ip_Address_[2]=W5100_Read_Byte(GATE_WAY_ADDRESS+2);
  _Remote_Ip_Address_[3]=W5100_Read_Byte(GATE_WAY_ADDRESS+3);
  return _Remote_Ip_Address_;
}


void UpdateSocketStatus()
{
   unsigned int RequestSize;
   unsigned int rp;
   char* rmac;
   unsigned int lp;
   char* rip;

   Socket0 =ReadStatusRegister();
   switch(Socket0)
    {
    //case socket is closed start intialize socket and start listening
    case SOCKET_CLOSED_STATUS:
    if (InitSocket(80)==0) {
    if (StartListening()==0);
    delay_ms(100);
    }
    break;

    //case a connection was accepted
    case SOCKET_ESTABLISHED_STATUS:
    RequestSize=GetRequestSize();
    if(RequestSize>0){
    rmac=GetRemoteMac();
    rip=GetRemoteIp();
    lp=GetLocalPort();
    rp=GetRemotePort();
    ProcessTcpRequest(rip,rmac,rp,lp,RequestSize);
    DisconnectSocket();}
    break;

    default:
    CloseSocket();
    break;
    }
}



void SendPage()
{
//command send to control register
W5100_Write_Byte(SOCKET0_CONTROL_REGISTER,SOCKET_CONTROL_REGISTER_SEND);
// wait untill control register is zero
while(W5100_Read_Byte(SOCKET0_CONTROL_REGISTER));
}